{-
   GlobalValues.hs
     - values used by multiple parts of program

   Author:       Martin Furman (s0830838)
   Organisation: University of Edinburgh
   Email:        mrt.furman@gmail.com
   Created:      22/11/2008
-}

module GlobalValues where


import Types

import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT

-- size of window
-- (one dimension is sufficient, as it is square)
mSize :: Int
mSize = 500

mGLsize :: GLsizei
mGLsize = fromIntegral (toInteger mSize)

-- title of window
mWindowTitle :: String
mWindowTitle = "Mandelbrot3D by Martin Furman"

-- color depth of rendering;
-- in the same time it represents the number of steps taken
-- to count mandelbrot step;
-- float used due to need to use floating point division
mColorDepth :: GLfloat
mColorDepth = fromIntegral (toInteger mSteps)
mSteps :: Int
mSteps = 512

-- resolution with which the mandelbrot set is displayed
--   - low resolution used to set preferences
--     for final rendering
--   - high resolution used for final rendering
mLowResolution, mHighResolution :: GLsizei
mHighResolution = 300
mLowResolution = 50

-- one step of rotation (in degrees)
mRotationStep :: GLfloat
mRotationStep = 5.0

-- one step of scaling (in percentage of actual scale)
mScaleStep :: GLfloat
mScaleStep = 0.1

-- one step of moving (changing offset) - (relative to scale)
mMoveStep :: GLfloat
mMoveStep = 0.1

-- maximum lift of one pixel (negative value)
mMaxLift :: GLfloat
mMaxLift = -0.05

