{-
   Main.hs
     - main module required as the the code is meant to be
       compiled and run as a standalone application because
       interpreter such as GHCi were not able to interprete
       the code correctly 

   Author:       Martin Furman (s0830838)
   Organisation: University of Edinburgh
   Email:        mrt.furman@gmail.com
   Created:      22/11/2008
-}

module Main where

import Types
import Area3D
import Display
import GlobalValues

import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT
import Data.IORef

main :: IO ()
main = do
  (progName, _) <- getArgsAndInitialize

  -- This is causing problems due to the bug in freeGLUT implementation combined
  -- with a computer not capable of direct rendering (hardware acceleration),
  -- thus the next line is commented out after application of the patch.
  -- initialDisplayMode $= [DoubleBuffered]

  -- init. of window (mGLsize defined in GlobalValues module)
  initialWindowSize $= Size mGLsize mGLsize
  createWindow mWindowTitle

  -- the ref. for initial area (see area function after where clause)
  area3D <- newIORef area

  -- the matrix used for rotation of the universe
  currentMatrix <- get ((matrix $ Just $ Modelview 0) :: StateVar (GLmatrix GLdouble))
  rotationMatrix <- newIORef currentMatrix

  -- callback for drawing and keyboard interaction (defined in Display module)
  displayCallback $= display area3D rotationMatrix
  keyboardMouseCallback $= Just (keyboardMouse area3D rotationMatrix)

  -- after all initial operations done
  -- enter main loop
  mainLoop

    where
    -- the initial area (the setting're the same as keyboard short-cut 0)
    -- see Area3D data type definition in Types module
    area = Area3D {
		 quality = LowQual,
		 offsetX = (-0.7),
		 offsetY = 0.0,
		 viewScale = 1.5,
		 viewRotation = mRotation,
		 isChanged = True
		 }
    
    -- the initial (empty) rotation
    -- see MRotation data type definition in Type module
    mRotation = MRotation {
		   rotationX = 0.0,
		   rotationY = 0.0,
		   rotationZ = 0.0
		   }
-- the end: main

