{-
   Spectrum.hs
     - spectrum color generation module

   Author:       Martin Furman (s0830838)
   Organisation: University of Edinburgh
   Email:        mrt.furman@gmail.com
   Created:      22/11/2008
-}


module Spectrum where

import Graphics.Rendering.OpenGL

-- this spectrum generating function is an approximation of visible light spectrum
-- i from 0.0 to 1.0: black -> violet -> blue -> cyan -> green -> yellow -> red
-- if i is more than 1.0 the function returns pure red, if less than 0.0 returns black
getSpectrum :: GLfloat -> Color3 GLfloat
getSpectrum i | i < s 1   = Color3 (up i 0.0)     0.0          (up i 0.0)
              | i < s 2   = Color3 (down i (s 1)) 0.0           1.0
              | i < s 3   = Color3  0.0          (up i (s 2))   1.0
              | i < s 4   = Color3  0.0           1.0          (down i (s 3))
              | i < s 5   = Color3 (up i (s 4))   1.0           0.0
              | otherwise = Color3  1.0          (down i (s 5)) 0.0
                  where
		  s :: GLfloat -> GLfloat
                  s = (*) (1.0 / 6.0)

		  up, down :: GLfloat -> GLfloat -> GLfloat
                  up i offset = (i - offset) / (s 1)
                  down i offset = 1 - (up i offset)
-- the end: getSpectrum

