{-
   Types.hs
     - definition of types used across the program

   Author:       Martin Furman (s0830838)
   Organisation: University of Edinburgh
   Email:        mrt.furman@gmail.com
   Created:      22/11/2008
-}

module Types where

import Graphics.Rendering.OpenGL
import Graphics.UI.GLUT

-- the quality (basically resolution) of the
-- final picture rendered
data MQuality = LowQual
              | HighQual
	      deriving (Eq, Ord)

-- the description of the final area to be rendered (quality,
-- scale, offset), rotation difference, and flag of change after
-- the last rendering
data Area3D = Area3D {
		  quality :: MQuality,
		  offsetX :: GLfloat,
		  offsetY :: GLfloat,
		  viewScale :: GLfloat,
		  viewRotation :: MRotation,
		  isChanged :: Bool
		  }

-- the rotation of next rendered frame relative to actual rotation
-- thus if the absolute rotation of 3D environment is the same as
-- the one on previous frame then all values are zero
data MRotation = MRotation {
			rotationX :: GLfloat,
			rotationY :: GLfloat,
			rotationZ :: GLfloat
			}

-- the representation of vertex as the triple of GLfloat
-- so it is easier to make operations with it
type MVertex = (GLfloat,GLfloat,GLfloat)

-- the representation of point as a tuple (no need to edit color
-- once defined (and evalueated due to immediate evaluation in
-- definition of Color3))
type MPoint = (MVertex,Color3 GLfloat)

-- simply the row of points
type MRow = [MPoint]

-- whole field is just the list of rows
type MArea = [MRow]

